#ifndef __ANELEX_WIRING_H__
#define __ANELEX_WIRING_H__

/**
 * ANELEX interface microcontroller
 * Wiring constants
 *
 * Defined symbols for the wiring of the uC on our board.
 * This is *very* specific to our hardware setup.
 *
 * This file is part of the Bull Anelex Project
 * Copyright (C) 2010  Sven Koeppel
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 3 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see
 * <http://www.gnu.org/licenses/>.
 *
 *
 **/

#include "avr/io.h"
#include "avr/interrupt.h"

// Port Input B: Button, Communication Announcment, IB2
#define PINB_BUT	PINB4
#define PINB_CA	    PINB2
#define PINB_IB2	PINB3
#define is_button()  (!( PINB & (1 << PINB_BUT) ))

// Port Input C: Errors (not important)
#define PINC_TA3	PINC7
#define PINC_TA1	PINC5 // TA2 was removed
#define PINC_TA0	PINC4

// Port Input D:
#define PIND_IB1    PIND2
#define PIND_LR     PIND3 // Line Ready
#define PIND_RTS    PIND4
// leider ist RTS nicht das, wofuer ich es gehalten habe: Es geht
// auf 1, wenn am PC ein terminal lauscht, sonst auf 0. Also keine
// "Ich habe noch Zeichen im Puffer"-Anzeige.
#define is_ready()  ( PIND & (1 << PIND_LR) )
#define is_rts()    ( PIND & (1 << PIND_RTS) )
#define has_character()  ( UCSR0A & 0b10000000 )

// Port Output A: DATA
#define set_data(x)      ( PORTA = ~ (x) )

// Port Output B: LED
#define PORTB_LED	     PORTB0
#define start_led()		 ( PORTB &= ~(1 << PORTB_LED) )
#define stop_led()		 ( PORTB |= (1 << PORTB_LED) )
#define is_led()         ( PINB & (1 << PORTB_LED) )
#define toggle_led()     ( is_led() ? stop_led() : start_led() )

// Port Output C: "Booleans Out"
#define PORTC_RESET      PORTC3
#define PORTC_PRINTCMD   PORTC2
#define PORTC_CP         PORTC1
#define PORTC_STROBE     PORTC0
#define start_strobe()   ( PORTC &= ~(1 << PORTC_STROBE) )  // auf 0 legen
#define stop_strobe()    ( PORTC |= (1 << PORTC_STROBE) )   // auf 1 heben
#define start_printcmd() ( PORTC &= ~(1 << PORTC_PRINTCMD) )
#define stop_printcmd()  ( PORTC |= (1 << PORTC_PRINTCMD) )

// Port Output D: Apperatus Indicators, flow control
#define PORTD_CTS        PORTD7
#define start_cts()		 ( PORTD &= ~(1 << PORTD_CTS) )
#define stop_cts()		 ( PORTD |= (1 << PORTD_CTS) )


#endif /* __ANELEX_WIRING_H__ */
