/*
 * Gamma 10 Interface Microcontroller
 * Test program level
 *
 */

#include "toolchain.h"

void Initialize()
{
	PORTA = 0x00;
	PORTB = 0x00; // LED (auslassen, an=0x01)
	PORTC = 0x00;
	PORTD = 0x00;

	DDRA = 0x00;
	DDRB = (1 << DDB0) | (1 << DDB1); // LED + RZMP1 als output
	DDRC = 0x00; // RMP1 Zeilenvorschuebe alle als Input
	DDRD = 0x00;

	// Pin Change Intterupts aufsetzen
	// PCMSK*: Pin Change Masks (welche Pins tragen zum Intterupt bei)
	// PORT C:
	PCMSK2 = (1 << PCINT16) | (1 << PCINT17) | (1 << PCINT18) | (1 << PCINT19);
	// Pin Change Interrupt Control Register: PC Interrupt Enable for Port C
	PCICR = (1 << PCIE2);

	// Interrupt enable
	sei();
}

// und hier der Interrupthandler fuer Port C:
int counter;
void foo();

// Negierte Logik beim Ausgang
#define stop_rzmp()		( PORTB &= ~(1 << PORTB1) )
#define start_rzmp()    ( PORTB |= (1 << PORTB1) )

ISR(PCINT2_vect) {
	// Bei Aenderung auf RMPx
	int value = PINC & 0x0F; // maskieren
	if(value == 0) return;   // nur was tun, wenn relevante PINC-Werte ungleich 0 sind
	
	// 5 Mikrosekunden warten
	_delay_us(5);
	// jetzt die Ausgabe "fertig" machen
	start_rzmp(); start_led();
	// signal muss min 7usec anliegen
	_delay_us(10);
	// Portb nullen
	stop_rzmp();
	stop_led();

	// fertig. Mal debugmaessig ausgeben, wie viel wir gemacht haben
	counter++;
	printf("%d: Zeilenvorschuebe %d = 0x%x = ", counter, value);
	print_bits(value);
}

void foo() {
	// PINC ist unser Freund.
//	printf("%d: %x = %d = ", counter, PINC, PINC);
//	print_bits(PINC);
/*
	// lightweight "." printing
	loop_until_bit_is_set(UCSR0A, UDRE0);
	UDR0 = '.';
*/

	// blinken lassen
	if (PINB & 0b00000001)	PORTB = PORTB & 0b11111110;
							else	PORTB = PORTB | 0b00000001;

	//_delay_ms(2000); // zwei sekunden warten

	// JETZT  die Ausgabe "fertig" machen
	//PORTB = (1 << PORTB1) | (1 << PORTB0);

	// dieses Signal muss min 7usec anliegen. Mache 10usec.
	//_delay_us(10);
	//_delay_ms(200); // damit mans sieht

	//PORTB = 0x00;

	counter++;
}


int main()
{
	counter = 0; // globaler Counter

	Initialize();
	UartInit(38400);
	setup_stdio();

	stop_led();

	byte_t x;
	for(x=0;;x++) {
		// schauen ob BUTTON gedrueckt ist
		if(PINB & (1 << PORTB4)) {
			start_led(); start_rzmp();
			for(;;) {
				if((PINB & (1 << PORTB4)) == 0)
					break;
			}
			printf("%d: Button gedrueckt\n", counter);
			stop_led(); stop_rzmp();
		}
		printf("%d: Sleeping...\n", counter);
		counter++;
		_delay_ms(1000);
		/*
		// das hier fuer ne endlostestschleife 2sec 
		// nix machen.
		byte_t v = (x%2) ? 0 : 1;
		puts(v ? "Jumping to 1" : "Jumping to 0");
		if(v) {
			start_led(); start_rzmp();
		} else {
			stop_led(); stop_rzmp();
		}
		_delay_ms(5*1000);
		*/
		//puts("sleeping...");
		//_delay_ms(2500);
		//if (PINB & 0b00000001)	PORTB = PORTB & 0b11111110;
		//						else	PORTB = PORTB | 0b00000001;
	}

	return 0;
}
